#include <iostream>
#include <cstdlib>
#include <cmath>

//こいつの中に本体がいる
#include "RootManager.hh"

static const int N(100000);
static const double PI(3.14159265358);

int main()
{
  // 本質的にシングルトンである必要がないが、シングルトンで管理するとよいと思う。
  // 一つの関数内ならこれでもいい。
  RootManager* root = new RootManager();
  // 入れ物の宣言。これもシングルトンで管理しておくのがよい。
  Dmap* D1 = new Dmap();
  Dmap* D2 = new Dmap();
  Imap* I = new Imap();
  //自前のクラスの入れ物を作るには、RootManager.hhのtypedefを参考に
  //その型のDataMapをつくる。他と同様にtypedefしておけばよい
  //(例)
  //typedef DataMap<std::string, std::pair<MyClass, std::string> > MyClassmap;
  //MyClassmap* MCmap = new MyClassmap();

  root->Initialize();//初回はなくてもよい。RootManagerをRunごとに使いまわすときなど
  root->CreateTree("t1", "test tree1");
  root->CreateTree("t2", "test tree2");

  //単に乱数使いたいから
  srand(128);
  // Input関数で内部に保存していくので自分で変数を保持する必要はない。
  for(int i = 0; i < N; ++i){
    double phi(2.0*PI*drand48());
    double theta(acos(1.0-2.0*drand48()));
    double radius(10.0*pow(drand48(), 1.0/3.0));

    double x(radius*sin(theta)*cos(phi));
    double y(radius*sin(theta)*sin(phi));
    double z(radius*cos(theta));

    //新しく追加したいならこの様にボンボン作っていくだけ。
    //ここがRootManagerの最大の持ち味
    root->Input(I,  "i", i, "i/I");
    root->Input(I,  "ii", i*i, "ii/I");
    root->Input(D1, "phi", phi, "phi/D");
    root->Input(D1, "theta", theta, "theta/D");
    root->Input(D1, "radius", radius, "radius/D");
    root->Input(D1, "x", x, "x/D");
    root->Input(D1, "y", y, "y/D");
    root->Input(D1, "z", z, "z/D");
    root->Input(D2, "xy", x*y, "xy/D");
    root->Input(D2, "yz", y*z, "yz/D");
    root->Input(D2, "zx", z*x, "zx/D");
    root->Input(D2, "xyz", x*y*z, "xyz/D");
    root->Input(D2, "norm", sqrt(x*x+y*y+z*z), "norm/D");
    root->Input(I,  "inorm", (int)sqrt(x*x+y*y+z*z), "norm/I");

    //初回のみイベントループの一番最後に呼ぶ。二回以上呼んだ場合のエラー検出はしていない。treeに登録。
    //DataMapごとに一括登録なので、treeを分けるなら別のDataMapに入れておく。
    if(i == 0){
      root->CreateBranch("t1", D1);
      root->CreateBranch("t1", I);
      root->CreateBranch("t2", D2);
    }
    //お約束
    root->Fill();    
  }

  //注意：最後に書き込むのだが、ひとつのTreeを書き込めるのは一回まで。
  //二回書き込もうとすると、一度目に書いた時点で破棄されるようで不可。
  //なので、Writeした時点でそのtreeを破棄するようにした。
#if 0
  //ぜんぶ一括でおなじrootファイルに書き込むときはこれ。これを呼んだときは、
  // root->Write("t1")とかを個別に呼べないようになっている。
  root->CreateFile("test1.root", "recreate");
  root->Write();
#else
  //個別の場合
  root->CreateFile("test1.root", "recreate");
  root->Write("t1");//あるtreeだけ書き込みたいならこれ。書き込むと同時にt1は消される。

  // さらに別のrootファイルを開いてそれにtreeを書き込む。
  // rootの仕様が変で、Fileを作ると、そのディレクトリに移動したと考えることになっている。
  root->CreateFile("test2.root", "recreate");
  root->Write("t2");
#endif

  //シングルトンで管理して、そのデストラクタにでも書いておきたい。
  delete I;
  delete D1; 
  delete D2; 
  delete root;

  return 0;
}
